# encoding:utf-8

"""
@author: Assem Chelli
@contact:  assem.ch [at] gmail.com
@license:  GPL

"""

#:buckwalter code
BUCKWALTER2UNICODE = {"'": u"\u0621",  # hamza-on-the-line
                      "|": u"\u0622",  # madda
                      ">": u"\u0623",  # hamza-on-'alif
                      "&": u"\u0624",  # hamza-on-waaw
                      "<": u"\u0625",  # hamza-under-'alif
                      "}": u"\u0626",  # hamza-on-yaa'
                      "A": u"\u0627",  # bare 'alif
                      "b": u"\u0628",  # baa'
                      "p": u"\u0629",  # taa' marbuuTa
                      "t": u"\u062A",  # taa'
                      "v": u"\u062B",  # thaa'
                      "j": u"\u062C",  # jiim
                      "H": u"\u062D",  # Haa'
                      "x": u"\u062E",  # khaa'
                      "d": u"\u062F",  # daal
                      "*": u"\u0630",  # dhaal
                      "r": u"\u0631",  # raa'
                      "z": u"\u0632",  # zaay
                      "s": u"\u0633",  # siin
                      "$": u"\u0634",  # shiin
                      "S": u"\u0635",  # Saad
                      "D": u"\u0636",  # Daad
                      "T": u"\u0637",  # Taa'
                      "Z": u"\u0638",  # Zaa' (DHaa')
                      "E": u"\u0639",  # cayn
                      "g": u"\u063A",  # ghayn
                      "_": u"\u0640",  # taTwiil
                      "f": u"\u0641",  # faa'
                      "q": u"\u0642",  # qaaf
                      "k": u"\u0643",  # kaaf
                      "l": u"\u0644",  # laam
                      "m": u"\u0645",  # miim
                      "n": u"\u0646",  # nuun
                      "h": u"\u0647",  # haa'
                      "w": u"\u0648",  # waaw
                      "Y": u"\u0649",  # 'alif maqSuura
                      "y": u"\u064A",  # yaa'
                      "F": u"\u064B",  # fatHatayn
                      "N": u"\u064C",  # Dammatayn
                      "K": u"\u064D",  # kasratayn
                      "a": u"\u064E",  # fatHa
                      "u": u"\u064F",  # Damma
                      "i": u"\u0650",  # kasra
                      "~": u"\u0651",  # shaddah
                      "o": u"\u0652",  # sukuun
                      "`": u"\u0670",  # dagger 'alif
                      "{": u"\u0671",  # waSla
                      # extended here
                      "^": u"\u0653",  # Maddah
                      "#": u"\u0654",  # HamzaAbove

                      ":": "\u06DC",  # SmallHighSeen
                      "@": "\u06DF",  # SmallHighRoundedZero
                      "\"": "\u06E0",  # SmallHighUprightRectangularZero
                      "[": "\u06E2",  # SmallHighMeemIsolatedForm
                      ";": "\u06E3",  # SmallLowSeen
                      ",": "\u06E5",  # SmallWaw
                      ".": "\u06E6",  # SmallYa
                      "!": "\u06E8",  # SmallHighNoon
                      "-": "\u06EA",  # EmptyCentreLowStop
                      "+": "\u06EB",  # EmptyCentreHighStop
                      "%": "\u06EC",  # RoundedHighStopWithFilledCentre
                      "]": "\u06ED"  #

                      }

POS = {
    "N": (u"اسم", "Noun"),
    "PN": (u"اسم علم", "Proper noun"),
    "IMPN": (u"اسم فعل أمر", "Imperative verbal noun"),
    "PRON": (u"ضمير", "Personal pronoun"),
    "DEM": (u"اسم اشارة", "Demonstrative pronoun"),
    "REL": (u"اسم موصول", "Relative pronoun"),
    "ADJ": (u"صفة", "Adjective"),
    "NUM": (u"رقم", "Number"),
    "T": (u"ظرف زمان", "Time adverb"),
    "LOC": (u"ظرف مكان", "Location adverb"),
    "V": (u"فعل", "Verb"),
    "P": (u"حرف جر", "Preposition"),
    "EMPH": (u"لام التوكيد", "Emphatic lām prefix"),
    "IMPV": (u"لام الامر", "Imperative lām prefix"),
    "PRP": (u"لام التعليل", "Purpose lām prefix"),
    "CONJ": (u"حرف عطف", "Coordinating conjunction"),
    "SUB": (u"حرف مصدري", "Subordinating conjunction"),
    "ACC": (u"حرف نصب", "Accusative particle"),
    "AMD": (u"حرف استدراك", "Amendment particle"),
    "ANS": (u"حرف جواب", "Answer particle"),
    "AVR": (u"حرف ردع", "Aversion particle"),
    "CAUS": (u"حرف سببية", "Particle of cause"),
    "CERT": (u"حرف تحقيق", "Particle of certainty"),
    "COND": (u"حرف شرط", "Conditional particle"),
    "EQ": (u"حرف تسوية", "Equalization particle"),
    "EXH": (u"حرف تحضيض", "Exhortation particle"),
    "EXL": (u"حرف تفصيل", "Explanation particle"),
    "EXP": (u"أداة استثناء", "Exceptive particle"),
    "FUT": (u"حرف استقبال", "Future particle"),
    "INC": (u"حرف ابتداء", "Inceptive particle"),
    "INTG": (u"حرف استفهام", "Interogative particle"),
    "NEG": (u"حرف نفي", "Negative particle"),
    "PREV": (u"حرف كاف", "Preventive particle"),
    "PRO": (u"حرف نهي", "Prohibition particle"),
    "REM": (u"حرف استئنافية", "Resumption particle"),
    "RES": (u"أداة حصر", "Restriction particle"),
    "RET": (u"حرف اضراب", "Retraction particle"),
    "SUP": (u"حرف زائد", "Supplemental particle"),
    "SUR": (u"حرف فجاءة", "Surprise particle"),
    "VOC": (u"حرف نداء", "Vocative particle"),
    "INL": (u"حروف مقطعة", "Quranic initials")
}

POSclass = {
    "Nouns": ["N", "PN", "IMPN"],
    "Pronouns": ["DEM", "REL", "PRON"],
    "Nominals": ["ADJ", "NUM"],
    "Adverbs": ["T", "LOC"],
    "Verbs": ["V"],
    "Prepositions": ["P"],
    "lām Prefixes": ["EMPH", "IMPV", "PRP"],
    "Conjunctions": ["CONJ", "SUB"],
    "Particles": ["ACC", "AMD", "ANS", "AVR", "CAUS", "CERT", "COND", "EQ", "EXH", "EXL", "EXP", "FUT", "INC", "INTG",
                  "NEG", "PREV", "PRO", "REM", "RES", "RET", "SUP", "SUR", "VOC"],
    "Disconnected Letters": ["INL"]
}

PREFIXclass = {
    "determiner": ["Al+"],
    "preposition": ["bi+", "ka+", "ta+", "l:P+"],
    "future particle": ["sa+"],
    "vocative particle": ["ya+", "ha+"],
    "interrogative particle": ["A:INTG+"],
    "equalization particle": ["A:EQ+"],
    "conjunction": ["wa+", "f:CONJ+"],
    "resumption": ["w:P+"],
    "cause": ["f:CAUS+"],
    "emphasis": ["l:EMPH+"],
    "purpose": ["l:PRP+"],
    "imperative": ["l:IMPV+"],
    "--undefined--": ["A+", "fa+"]
}

PREFIX = {
    "Al+": (u"ال", u"al"),
    "bi+": (u"ب", u"bi"),
    "ka+": (u"ك", u"ka"),
    "ta+": (u"ت", u"ta"),
    "sa+": (u"س", u"sa"),
    "ya+": (u"يا", u"yā"),
    "ha+": (u"ها", u"hā"),
    "A+": (u"أ", u"alif"),
    "A:INTG+": (u"أ", u"alif"),
    "A:EQ+": (u"أ", u"alif"),
    "wa+": (u"و", u"wa"),
    "w:P+": (u"و", u"wa"),
    "fa+": (u"ف", u"fa"),
    "f:CONJ+": (u"ف", u"fa"),
    "f:REM+": (u"ف", u"fa"),
    "f:CAUS+": (u"ف", u"fa"),
    "l:P+": (u"ل", u"lām"),
    "l:EMPH+": (u"ل", u"lām"),
    "l:PRP+": (u"ل", u"lām"),
    "l:IMPV+": (u"ل", u"lām"),
}

PGNclass = {
    "person": ["1", "2", "3"],
    "number": ["S", "D", "P"],
    "gender": ["M", "F"]
}

PGN = {
    "1": u"متكلم",
    "2": u"مخاطب",
    "3": u"غائب",
    "M": u"مذّكر",
    "F": u"مؤنّث",
    "S": u"مفرد",
    "D": u"مثنّى",
    "P": u"جمع"
}

VERBclass = {
    "aspect": ["PERF", "IMPF", "IMPV"],
    "mood": ["IND", "SUBJ", "JUS", "ENG"],
    "voice": ["ACT", "PASS"],
    "form": ["(I)", "(II)", "(III)", "(IV)", "(V)", "(VI)", "(VII)", "(VIII)", "(IX)", "(X)", "(XI)", "(XII)"]
}

VERB = {
    "PERF": (u"فعل ماض", "Perfect verb"),
    "IMPF": (u"فعل مضارع", "Imperfect verb"),
    "IMPV": (u"فعل أمر", "Imperative verb"),
    "IND": (u"مرفوع", "Indicative mood"),
    "SUBJ": (u"منصوب", "Subjunctive mood"),
    "JUS": (u"مجزوم", "Jussive mood"),
    "ENG": (u"مؤكد", "Energetic mood"),
    "ACT": (u"مبني للمعلوم", "Active voice"),
    "PASS": (u"مبني للمجهول", "Passive voice"),
    "(I)": (u"", "First form"),
    "(II)": (u"", "Second form"),
    "(III)": (u"", "Third form"),
    "(IV)": (u"", "Fourth form"),
    "(V)": (u"", "Fifth form"),
    "(VI)": (u"", "Sixth form"),
    "(VII)": (u"", "Seventh form"),
    "(VIII)": (u"", "Eighth form"),
    "(IX)": (u"", "Ninth form"),
    "(X)": (u"", "Tenth form"),
    "(XI)": (u"", "Eleventh form"),
    "(XII)": (u"", "Twelfth form")
}

DERIVclass = {
    "derivation": ["ACT PCPL", "PASS PCPL", "VN"]
}

DERIV = {
    "ACT PCPL": (u"اسم فاعل", "Active participle"),
    "PASS PCPL": (u"اسم مفعول", "Passive participle"),
    "VN": (u"مصدر", "Verbal noun")
}

NOMclass = {
    "state": ["DEF", "INDEF"],
    "case": ["NOM", "ACC", "GEN"]
}

NOM = {
    "DEF": (u"معرفة", "Definite state"),
    "INDEF": (u"نكرة", "Indefinite state"),
    "NOM": (u"مرفوع", "Nominative case"),
    "ACC": (u"منصوب", "Accusative case"),
    "GEN": (u"مجرور", "Genitive case"),
}

PRON = {
    "*": {u"ني", u"نا", u"ك", u"كما", u"كم", u"ه", u"هما", u"هم", u"كن", u"ها", u"هن"},
    "1": {u"ني", u"نا"},
    "2": {u"ك", u"كما", u"كم", u"كن"},
    "3": {u"ه", u"ها", u"هما", u"هم", u"هن"},
    "M": {u"ني", u"نا", u"ك", u"كما", u"كم", u"ه", u"هما", u"هم"},
    "F": {u"ني", u"نا", u"ك", u"كما", u"كن", u"ها", u"هما", u"هن"},
    "S": {u"ني", u"ك", u"ه", u"ها"},
    "D": {u"نا", u"كما", u"هما"},
    "P": {u"نا", u"كم", u"هم", u"كن", u"هن"},
}



#!/usr/bin/env python2
# encoding:utf-8

"""
@author: Assem Chelli
@contact: assem.ch [at] gmail.com
@license: GPL
"""

# import libxml2#,xpath
import xml.etree.ElementTree
from pyparsing import Keyword, Word, Group, Literal, CharsNotIn, alphas
from pyparsing import SkipTo, ZeroOrMore, Optional, OneOrMore

from constants import BUCKWALTER2UNICODE
from constants import DERIV, PREFIX, PGN, POS, VERB, NOM, PRON
from constants import DERIVclass, NOMclass, PGNclass, VERBclass, PREFIXclass, POSclass


def reverse_class(dictionary):
    """ invert a dictionary """
    newdict = {}
    for key, value in dictionary.iteritems():
        if type(value) is not list:
            value = [value]
        for v in value:
            if v in newdict:
                newdict[v].append(key)
            else:
                newdict[v] = [key]
    return newdict


def buck2uni(string):
    """ decode buckwalter """
    result = ""
    for ch in string:
        result += BUCKWALTER2UNICODE[ch]
    return result


def tag_keywords(list_tags):
    """a specific pyparsing term to match tags return Keywords
     """
    res = list_tags[0]
    for item in list_tags[1:]:
        res |= Keyword(item)
    return res


def tag_literals(list_tags):
    """a specific pyparsing term to match tages return Literals"""

    res = list_tags[0]
    for item in list_tags[1:]:
        res |= Literal(item)
    return res

class MorphologyParser:
        """ parse the Morphology tags """

        def __init__(self):
            pass

        @staticmethod
        def parse_step1(morph):
            """parse the field morphology of qurany corpus

            """

            string = "$ " + str(morph).replace("POS:", "£ POS:") \
                .replace("PRON:", "µ PRON:") \
                .replace("&lt;", "<") \
                .replace("&gt;", ">") + " #"
            # regular expressions
            begin = Keyword('$').suppress()
            center = Keyword('£').suppress()
            last = Keyword('µ').suppress()
            end = Keyword('#').suppress()
            skip = SkipTo(end).suppress()

            prefix = Word(alphas + "+" + ":")
            prefixes = Group(ZeroOrMore(~center + prefix))

            genderK = tag_keywords(["M", "F"])
            numberK = tag_keywords(["S", "D", "P"])
            # personK = tag_keywords(["1", "2", "3"])

            genderL = tag_literals(["M", "F"])
            numberL = tag_literals(["S", "D", "P"])
            personL = tag_literals(["1", "2", "3"])

            person_ = personL + Optional(genderL) + Optional(numberL)
            gender_ = genderL + numberL

            gen = person_ | gender_ | numberK | genderK
            pos = "POS:" + Word(alphas)
            lem = "LEM:" + CharsNotIn(" ")
            root = "ROOT:" + CharsNotIn(" ")
            sp = "SP:" + CharsNotIn(" ")
            mood = "MOOD:" + CharsNotIn(" ")

            aspect = tag_keywords(["PERF", "IMPF", "IMPV"])

            voice = tag_keywords(["ACT", "PASS"])
            form = tag_keywords(
                ["(I)", "(II)", "(III)", "(IV)", "(V)", "(VI)", "(VII)", "(VIII)", "(IX)", "(X)", "(XI)", "(XII)"])
            verb = aspect | voice | form

            voc = Keyword("+voc").suppress()

            deriv = tag_keywords(["ACT", "PCPL", "PASS", "VN"])

            state = tag_keywords(["DEF", "INDEF"])
            case = tag_keywords(["NOM", "ACC", "GEN"])
            nom = case | state

            tag = lem | root | sp | mood | gen | verb | deriv | nom | voc | skip
            part = Group(center + pos + ZeroOrMore(~center + ~last + ~end + tag))

            base = Group(OneOrMore(~end + ~last + part))

            pron = "PRON:" + Group(gen)
            suffixes = Group(ZeroOrMore(~end + last + pron))

            whole = begin + prefixes + base + suffixes + end

            parsed = whole.parseString(string)

            return parsed

        @staticmethod
        def parse_step2(parsedlist):
            """ return a dict """
            Dict = {}
            # prefixes
            prefixes = parsedlist[0]
            Dict["prefixes"] = []
            if prefixes:
                for prefix in prefixes:
                    prefixDict = {
                        "token": PREFIX[prefix][1],
                        "arabictoken": PREFIX[prefix][0],
                        "type": reverse_class(PREFIXclass)[prefix][0]
                    }
                    Dict["prefixes"].append(prefixDict)

            # word base
            parts = parsedlist[1]
            Dict["base"] = []
            for part in parts:
                partDict = {}
                for i in range(len(part)):
                    tag = part[i]
                    if tag[-1] == ":":
                        nexttag = part[i + 1]
                        if tag == "POS:":
                            partDict["type"] = reverse_class(POSclass)[nexttag][0]
                            partDict["pos"] = POS[nexttag][1]
                            partDict["arabicpos"] = POS[nexttag][0]
                        elif tag == "ROOT:":
                            partDict["root"] = nexttag
                            partDict["arabicroot"] = buck2uni(nexttag)
                        elif tag == "LEM:":
                            partDict["lemma"] = nexttag
                            partDict["arabiclemma"] = buck2uni(nexttag)
                        elif tag == "SP:":
                            partDict["special"] = nexttag
                            partDict["arabicspecial"] = buck2uni(nexttag)
                        elif tag == "MOOD:":
                            partDict["mood"] = VERB[nexttag][1]
                            partDict["arabicmood"] = VERB[nexttag][0]
                        else:
                            print "new tag!! " + tag
                        i += 1
                    else:
                        if tag in PGN:
                            partDict[reverse_class(PGNclass)[tag][0]] = PGN[tag]
                        elif tag in ["ACT", "PASS"]:
                            nexttag = part[i + 1] if i + 1 < len(part) else None
                            if nexttag == "PCPL":
                                partDict[reverse_class(DERIVclass)[tag + " PCPL"][0]] = DERIV[tag + " PCPL"][1]
                                i += 1
                        elif tag in VERB:
                            partDict[reverse_class(VERBclass)[tag][0]] = VERB[tag][1]

                        elif tag in NOM:
                            def arabize(x):
                                return "arabicstate" if x == "state" else "arabiccase"

                            partDict[reverse_class(NOMclass)[tag][0]] = NOM[tag][1]
                            partDict[arabize(reverse_class(NOMclass)[tag][0])] = NOM[tag][0]

                        elif tag == "VN":
                            partDict[reverse_class(DERIVclass)[tag][0]] = DERIV[tag][1]

                Dict["base"].append(partDict)

            # suffixes
            suffixes = parsedlist[2]
            Dict["suffixes"] = []
            if suffixes:
                for i in range(len(suffixes)):
                    tag = suffixes[i]
                    if tag == "PRON:":
                        pronDict = {}
                        Pset = set(PRON["*"])
                        pronprops = suffixes[i + 1]
                        for tag in pronprops:
                            if tag in PGN:
                                pronDict[reverse_class(PGNclass)[tag][0]] = PGN[tag]
                                Pset &= PRON[tag]

                        pronDict["arabictoken"] = Pset.pop() if Pset else ""
                        Dict["suffixes"].append(pronDict)

            return Dict

        @staticmethod
        def parse(string):
            return MorphologyParser.parse_step2(MorphologyParser.parse_step1(string))



class API:
    def __init__(self, source="./data/quranic-corpus-morpology.xml"):
        """
        init the API based on XMLfile
        @param source: the path of the xml file
        """
        self.corpus = xml.etree.ElementTree.parse(source)
        # libxml2.parseFile(source)
        # print xpath.find('//item', source)


    def unique_words(self):
        """return a dictionary: the keys is word tokens and the values is the properties"""
        D = {}
        for chapter in self.corpus.findall(".//chapter"):
            for verse in chapter.findall("verse"):
                for word in verse.findall("word"):
                    D[word.attrib["token"]] = MorphologyParser.parse(word.attrib["morphology"])
        return D

    def all_words_generator(self):
        """
        Generate words properties ,word by word
        """
        for chapter in self.corpus.findall(".//chapter"):
            for verse in chapter.findall("verse"):
                for word in verse.findall("word"):
                    res = word.attrib
                    res["sura_id"] = int(chapter.attrib["number"])
                    res["aya_id"] = int(verse.attrib["number"])
                    res["word_id"] = int(word.attrib["number"])
                    res["word"] = word.attrib["token"]
                    res["morphology"] = MorphologyParser.parse(word.attrib["morphology"])
                    yield res


if __name__ == "__main__":
    A = API(source="data/quranic-corpus-morpology.xml")
    # for item in A.all_words_generator():
    #     print "(sura,aya,word):({0},{1},{2})".format(item["sura_id"], item["aya_id"], item["word_id"])

    # for item in A.unique_words():
    #      print item

    # print MorphologyParser.parse("fa+ POS:INTG LEM:&lt;maA ROOT:qawol l:P+ ")
    # print A.corpus.findtext("@number=’114’")



#!/usr/bin/env python2
# coding = utf-8


from setuptools import setup

setup(
    name="qurancorpus",
    description="Arabic Quranic Corpus python API",
    version=0.1,
    platforms="ALL",
    license="GPL",
    packages=["qurancorpus"],
    install_requires=['pyparsing'],

    author="Assem Chelli",
    author_email="assem.ch@gmail.com",
    package_dir={'qurancorpus': '.'},
    long_description="""A python api for the Quranic Arabic Corpus project""",
    keywords="quran arabic corpus quranic",

    include_package_data=True,

    data_files=[ ('./qurancorpus/data/',['data/quranic-corpus-morpology.xml'])],

    zip_safe=True,

    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: Arabic",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
